use std::env;

use chrono::{Local, NaiveDateTime};
use lettre::{
    message::header::ContentType, transport::smtp::authentication::Credentials, Message,
    SmtpTransport, Transport,
};
use rust_decimal::{prelude::ToPrimitive, Decimal};
use sqlx::{Connection, MySqlConnection, Row};

#[tokio::main]
async fn main() {
    dotenvy::dotenv().ok();
    let mut connection = MySqlConnection::connect(
        &env::var("DATABASE_URL").expect("DATABASE_URL environment variable"),
    )
    .await
    .unwrap();

    let email_host = env::var("EMAIL_HOST").expect("EMAIL_HOST env var");
    let email_user = env::var("EMAIL_USER").expect("EMAIL_USER env var");
    let email_pass = env::var("EMAIL_PASS").expect("EMAIL_PASS env var");
    let email_name = env::var("EMAIL_NAME").expect("EMAIL_NAME env var");
    let email_to = env::var("EMAIL_TO").expect("EMAIL_TO env var");

    let row =
        sqlx::query("SELECT timestamp, value FROM counter_value ORDER BY timestamp DESC LIMIT 1")
            .fetch_one(&mut connection)
            .await
            .unwrap();

    let timestamp: NaiveDateTime = row.get("timestamp");
    let value: Decimal = row.get("value");

    let now = Local::now();

    let mut email =
        Message::builder().from(format!("{email_name} <{email_user}>").parse().unwrap());
    for to in email_to.split(',') {
        email = email.to(to.parse().unwrap());
    }
    let email = email
        .subject(format!("Zählerstand vom {}", now.format("%d.%m.%Y")))
        .header(ContentType::TEXT_PLAIN)
        .body(format!(
            "Zählerstand: {:.3} kWh\nStand: {} UTC",
            (value.to_u128().unwrap() as f64) / 1000.0,
            timestamp.format("%d.%m.%Y %H:%m"),
        ))
        .unwrap();

    let creds = Credentials::new(email_user, email_pass);

    SmtpTransport::relay(&email_host)
        .unwrap()
        .credentials(creds)
        .build()
        .send(&email)
        .unwrap();
}
